/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "html/head.hh"
#include "html/writer.hh"
#include <iostream>
#include <unistd.h>

using namespace html;
using namespace std;

//===================================  header info class
head::head(void) {
}

head*
head::clone(void) const {
   return new head(*this);
}

void
head::write(writer& w) const {
   if (!w.empty_line()) w.endLine();
   compound::write(w);
   w.endLine();
}

//===================================  Related data link class
rel_link::rel_link(const string& href, const string& rel, const string& type) 
   : tag_only("link")
{
   addAttr("href", href);
   addAttr("rel",  rel);
   addAttr("type", type);
}

//===================================  Related data link class
rel_link*
rel_link::clone(void) const {
   return new rel_link(*this);
}

//======================================  Install local script copy.
void
rel_link::install(const char* from, const char* to) const {
   if (attributes().exists("href")) {
      string src_file = attributes().find("href").valString();
      if (!src_file.empty() &&
	  src_file.find("://") == string::npos &&
	  src_file[0] != '/')
      {
	 string target = to;
	 if (!target.empty() && target[target.size()-1] != '/') target += "/";
	 target += src_file;
	 if (!access(target.c_str(), R_OK)) return; //  bail if file extsts.
	 string source = from;
	 if (!source.empty() && source[source.size()-1] != '/') source += "/";
	 source += src_file;
	 string cmd = string("/bin/cp -avu ") + source + " " + target;
	 int rc = system(cmd.c_str());
	 if (rc < 0) {
	    cerr << "html::import: unable to install " << source
		 << " in " << to << endl; 
	 }
      }
   }
}


//===================================  Head metadata class
meta::meta(const string& name, const string& content) 
   : tag_only("meta")
{
   addAttr("name",    name);
   addAttr("content", content);
}

//===================================  Head metadata class
meta*
meta::clone(void) const {
   return new meta(*this);
}

//====================================  Title class
title::title(const string& doc_ttl) 
   : text_block("title")
{
   addText(doc_ttl);
}
